CREATE TABLE Person (
	pid             INTEGER PRIMARY KEY,
	vname           VARCHAR(50) NOT NULL,
	nname           VARCHAR(50) NOT NULL,
	gebdat          DATE NOT NULL,
	adresse         VARCHAR(100) NOT NULL,
	
	CONSTRAINT person_eindeutig UNIQUE (vname, nname, gebdat, adresse));

CREATE TABLE Kunde (
	pid				INTEGER PRIMARY KEY REFERENCES Person);

CREATE TABLE Mitarbeiter (
	pid             INTEGER PRIMARY KEY REFERENCES Person,
	svnr            NUMERIC(4) NOT NULL,
	eindat          DATE NOT NULL,
	gehalt          NUMERIC(7,2) NOT NULL,
	kontonr         NUMERIC(15) NOT NULL,
	blz             NUMERIC(6) NOT NULL,
	zweigstelle     VARCHAR(50) NOT NULL);

CREATE TABLE Zweigstelle (
	name            VARCHAR(50) PRIMARY KEY,
	anschrift       VARCHAR(100) NOT NULL,
	telefon         VARCHAR(50) NOT NULL,
	email           VARCHAR(50) NOT NULL,
	leiter          INTEGER NOT NULL REFERENCES Mitarbeiter (pid) DEFERRABLE INITIALLY DEFERRED);

ALTER TABLE Mitarbeiter ADD CONSTRAINT fk_zweigstelle
	FOREIGN KEY (zweigstelle) REFERENCES Zweigstelle (name)
	DEFERRABLE INITIALLY DEFERRED;

CREATE TABLE Medien (
	mediennr        INTEGER PRIMARY KEY,
	jahr            NUMERIC(4) NOT NULL,
	titel           VARCHAR(100) NOT NULL,
	freigabe        NUMERIC(2) DEFAULT '0' NOT NULL);

CREATE TABLE Themengebiet (
	themenid        INTEGER PRIMARY KEY,
	name            VARCHAR(50) NOT NULL);

CREATE TABLE untergeordnet (
	unterthema      INTEGER PRIMARY KEY REFERENCES Themengebiet (themenid),
	oberthema       INTEGER NOT NULL REFERENCES Themengebiet (themenid),
	
	CHECK (unterthema <> oberthema));

CREATE TABLE zugeordnet (
	mediennr        INTEGER REFERENCES Medien,
	themenid        INTEGER REFERENCES Themengebiet,
	
	PRIMARY KEY(mediennr, themenid));

CREATE TABLE Kuenstler (
	kid             INTEGER PRIMARY KEY,
	vname           VARCHAR(50) NOT NULL,
	nname           VARCHAR(50) NOT NULL);

CREATE TABLE Buecher (
	mediennr        INTEGER PRIMARY KEY REFERENCES Medien,
	umfang          INTEGER NOT NULL);

CREATE TABLE Filme (
	mediennr        INTEGER PRIMARY KEY REFERENCES Medien,
	format          CHAR(3) NOT NULL CHECK (format IN ('8mm', 'VHS', 'DVD')),
	regisseur       INTEGER NOT NULL REFERENCES Kuenstler (kid));

CREATE TABLE verfasst (
	kid             INTEGER REFERENCES Kuenstler,
	mediennr        INTEGER REFERENCES Medien,
	
	PRIMARY KEY(kid, mediennr));

CREATE TABLE Exemplar (
	mediennr        INTEGER REFERENCES Medien,
	enr             INTEGER,
	kaufdat         DATE NOT NULL,
	zweigstelle     VARCHAR(50) NOT NULL REFERENCES Zweigstelle (name),
	
	PRIMARY KEY(mediennr, enr));

CREATE TABLE Entlehnkarte (
	nr              INTEGER PRIMARY KEY,
	gueltig         DATE NOT NULL,
	gebuehr         NUMERIC(6,2) NOT NULL,
	person          INTEGER NOT NULL REFERENCES Kunde (pid));

CREATE TABLE Entlehnung (
	eid             INTEGER PRIMARY KEY,
	adat            DATE NOT NULL,
	fdat            DATE NOT NULL,
	entlehnkarte    INTEGER NOT NULL REFERENCES Entlehnkarte (nr));

CREATE TABLE entlehnt (
	eid             INTEGER REFERENCES Entlehnung,
	mediennr        INTEGER,
	enr             INTEGER,
	
	PRIMARY KEY(eid, mediennr, enr),
	FOREIGN KEY(mediennr, enr) REFERENCES Exemplar);
	
CREATE TABLE retourniert (
	eid             INTEGER REFERENCES Entlehnung,
	mediennr        INTEGER,
	enr             INTEGER,
	zweigstelle		VARCHAR(50) NOT NULL REFERENCES Zweigstelle (name),
	rdat			DATE NOT NULL,
		
	PRIMARY KEY (eid, mediennr, enr),
	FOREIGN KEY (mediennr, enr) REFERENCES Exemplar);


CREATE SEQUENCE entlehnkarte_sequence
START WITH 1000000000
INCREMENT BY 15
MINVALUE 1000000000
MAXVALUE 9999999990
NO CYCLE;

CREATE SEQUENCE medien_sequence;

CREATE SEQUENCE kuenstler_sequence;
